/*
** File: linux-2.6.c  
** Project: Linux ADB3 monolithic driver
** Purpose: Entry point for kernel module and other Linux-specific declarations.
**
** (C) Copyright Alpha Data 2010-2012
*/

#include <linux/module.h>
#include <linux/init.h>

#include "df.h"
#include "dflinux.h"

static const char* g_pciDriverName = "adb3";

MODULE_LICENSE("GPL");

/*
** Module parameters
*/

static uint AssumeXrmPresent = 0;
module_param(AssumeXrmPresent, uint, S_IRUGO);
static uint DebugLevel = 0;
module_param(DebugLevel, uint, S_IRUGO);
static uint DmaMapLength = 64;
module_param(DmaMapLength, uint, S_IRUGO);
static uint EnableVpdWrite = 0;
module_param(EnableVpdWrite, uint, S_IRUGO);
static uint IncludeDiagSensors = 0;
module_param(IncludeDiagSensors, uint, S_IRUGO);
static uint LegacyHardware = 0;
module_param(LegacyHardware, uint, S_IRUGO);
static uint PciAddress64Bit = 0;
module_param(PciAddress64Bit, uint, S_IRUGO);
static uint Si5338ExposeAllClocks = 0;
module_param(Si5338ExposeAllClocks, uint, S_IRUGO);
static uint PrimaryCommonBufferCount = 0;
module_param(PrimaryCommonBufferCount, uint, S_IRUGO);
static uint PrimaryCommonBufferAlignment = 16U;
module_param(PrimaryCommonBufferAlignment, uint, S_IRUGO);
static uint PrimaryCommonBufferSizeLow = 0x10000U;
module_param(PrimaryCommonBufferSizeLow, uint, S_IRUGO);
static uint PrimaryCommonBufferSizeHigh = 0;
module_param(PrimaryCommonBufferSizeHigh, uint, S_IRUGO);

DfLinuxDriverParameterUint32 g_dfLinuxDriverParamUint32[] = {
  { "DebugLevel", &DebugLevel },
  { "DmaMapLength", &DmaMapLength },
  { "PrimaryCommonBufferCount", &PrimaryCommonBufferCount },
  { "PrimaryCommonBufferAlignment", &PrimaryCommonBufferAlignment },
  { "PrimaryCommonBufferSizeLow", &PrimaryCommonBufferSizeLow },
  { "PrimaryCommonBufferSizeHigh", &PrimaryCommonBufferSizeHigh },
  { NULL, NULL } /* Terminator */
};
  
DfLinuxDriverParameterBoolean g_dfLinuxDriverParamBoolean[] = {
  { "AssumeXrmPresent", &AssumeXrmPresent },
  { "EnableVpdWrite", &EnableVpdWrite },
  { "IncludeDiagSensors", &IncludeDiagSensors },
  { "LegacyHardware", &LegacyHardware },
  { "PciAddress64Bit", &PciAddress64Bit },
  { "Si5338ExposeAllClocks", &Si5338ExposeAllClocks },
  { NULL, NULL } /* Terminator */
};
  
DfLinuxDriverParameterString g_dfLinuxDriverParamString[] = {
  { NULL, NULL } /* Terminator */
};
  
/*
** PCI ID table used by hotplug subsystem. The table is in two parts, where the second part is for legacy ID.
** By putting NULL ID after the first part of the table, legacy hardware support is NOT advertised to the
** hotplug subsystem (intentional).
**
** If TRUE is passed in bSupportLegacyHardware, dfLinuxModuleInitPci() function removes the NULL ID after the
** first part and shifts the second part entries down, so that legacy hardware will be registered when the driver
** calls pci_register_driver.
**
** !!!NOTE!!! If there is no legacy hardware, the table MUST end with two NULL PCI ID records, e.g.
**
** static struct pci_device_id g_pciIdTable[] = {
**   { .vendor = ..., .device = ..., .subvendor = ..., .subdevice = ... },
**   { .vendor = ..., .device = ..., .subvendor = ..., .subdevice = ... },
**   { .vendor = 0x0U, .device = 0x0U },
**   { .vendor = 0x0U, .device = 0x0U }
** }
**
** In other words, if there is no legacy hardware, then do not remove the NULL ID for terminating the legacy
** hardware part of the table.
*/
static struct pci_device_id g_pciIdTable[] = {
  { .vendor = 0x4144U, .device = 0xadb3U, .subvendor = 0x4144U, .subdevice = PCI_ANY_ID }, /* ADB3-based models */
  { .vendor = 0x0U, .device = 0x0U }, /* Marks end of non-legacy hardware */
  /* Legacy hardware begins here */
  { .vendor = 0x10b5U, .device = 0x9656U, .subvendor = 0x4144U, .subdevice = PCI_ANY_ID }, /* PCI9656-based models */
  { .vendor = 0x0U, .device = 0x0U }  /* Marks end of table */
};

#if 0
/* When we have sorted out udev, we can enable the MODULE_DEVICE_TABLE so that the module loads automatically instead of via adb3.rc script */
MODULE_DEVICE_TABLE(pci, g_pciIdTable);
#endif

/*
** Module entry / exit points
*/

int __init
adb3Init(
  void)
{
#if DF_DBG_BUILD
  g_dfDebugLevel = (int)DebugLevel;
#endif
  return dfLinuxModuleInitPci(g_pciDriverName, g_pciIdTable, LegacyHardware ? TRUE : FALSE);
}

void __exit
adb3Exit(
  void)
{
  dfLinuxModuleExitPci();
}

module_init(adb3Init)
module_exit(adb3Exit)
